#include <amxmodx>
#include <fakemeta>
#include <xs>

#define PLUGIN_NAME "Fired Particle Details"
#define PLUGIN_VERSION "0.1"
#define PLUGIN_AUTHOR "VEN"

#define MAX_BURST_FIRE_TIME 0.2

#define GUNS_BIT_SUM ((1<<CSW_P228) | (1<<CSW_SCOUT) | (1<<CSW_XM1014) | (1<<CSW_MAC10) | (1<<CSW_AUG) | (1<<CSW_ELITE) | (1<<CSW_FIVESEVEN) | (1<<CSW_UMP45) | (1<<CSW_SG550) | (1<<CSW_GALIL) | (1<<CSW_FAMAS) | (1<<CSW_USP) | (1<<CSW_GLOCK18) | (1<<CSW_AWP) | (1<<CSW_MP5NAVY) | (1<<CSW_M249) | (1<<CSW_M3) | (1<<CSW_M4A1) | (1<<CSW_TMP) | (1<<CSW_G3SG1) | (1<<CSW_DEAGLE) | (1<<CSW_SG552) | (1<<CSW_AK47) | (1<<CSW_P90))
#define BURSTGUNS_BIT_SUM ((1<<CSW_FAMAS) | (1<<CSW_GLOCK18))
#define SHOTGUNS_BIT_SUM ((1<<CSW_XM1014) | (1<<CSW_M3))

#define MAX_PLAYERS 32

new g_normal_ptr[MAX_PLAYERS + 1]
new Float:g_last_shotgun_shot_gametime[MAX_PLAYERS + 1]
new g_shotgun_shot_particle_number[MAX_PLAYERS + 1]
new Float:g_last_attack_release_gametime[MAX_PLAYERS + 1]

public plugin_init() {
	register_plugin(PLUGIN_NAME, PLUGIN_VERSION, PLUGIN_AUTHOR)
	register_forward(FM_TraceLine, "fwTraceLine", 1) // post hook so we can access a trace result
	register_forward(FM_PlayerPreThink, "fwPlayerPreThink")
}

public client_disconnect(id) {
	// clear variables on disconnect
	g_normal_ptr[id] = 0
	g_last_shotgun_shot_gametime[id] = 0.0
	g_shotgun_shot_particle_number[id] = 0
	g_last_attack_release_gametime[id] = 0.0
}

public fwPlayerPreThink(id) {
	if ((pev(id, pev_oldbuttons) & IN_ATTACK) && !(pev(id, pev_button) & IN_ATTACK)) {
		// the primary attack button is released

		// save the current gametime
		g_last_attack_release_gametime[id] = get_gametime()
	}
}

public fwTraceLine(const Float:start[3], const Float:dest[3], ignore_monsters, id, ptr) {
	if (ignore_monsters) {
		// trace is ignore a monsters

		// this is not a fired particle trace

		return FMRES_IGNORED
	}

	if (!is_user_alive(id)) {
		// the player isn't alive

		// he can't perform a shot

		// this is not a fired particle trace

		return FMRES_IGNORED
	}

	if (!g_normal_ptr[id]) {
		// normal trace isn't fired yet

		// this is a normal trace

		// save the normal trace pointer
		g_normal_ptr[id] = ptr

		return FMRES_IGNORED
	}

	if (ptr == g_normal_ptr[id]) {
		// this is a normal trace

		// this is not a fired particle trace

		return FMRES_IGNORED
	}

	static clip, ammo, weapon_id

	// get the current weapon index
	weapon_id = get_user_weapon(id, clip, ammo)

	if (!(GUNS_BIT_SUM & (1<<weapon_id))) {
		// the current weapon isn't a gun

		// this is not a fired particle trace

		return FMRES_IGNORED
	}

	static Float:gametime

	// get current game time
	gametime = get_gametime()

	new bool:is_hold_primary_attack = bool:(pev(id, pev_button) & IN_ATTACK)

	if (!(BURSTGUNS_BIT_SUM & (1<<weapon_id))) {
		// the current weapon isn't a burst gun

		if (!is_hold_primary_attack) {
			// player doesn't hold a primary attack button

			// this is not a fired particle trace

			return FMRES_IGNORED
		}
	}
	else {
		// the current weapon is a burst gun

		if (!is_hold_primary_attack) {
			// player doesn't hold a primary attack button

			if (gametime - g_last_attack_release_gametime[id] > MAX_BURST_FIRE_TIME) {
				// player haven't used the primary attack button at least for a MAX_BURST_FIRE_TIME second

				// this is not a fired particle trace

				return FMRES_IGNORED
			}
		}
	}

	static Float:fired_particle_start_origin[3], Float:vector[3]
	
	// get the player's origin and view offset
	pev(id, pev_origin, fired_particle_start_origin)
	pev(id, pev_view_ofs, vector)

	// get the fired particle start origin
	xs_vec_add(fired_particle_start_origin, vector, fired_particle_start_origin)

	static info[256], fired_particle_type[24]

	// set fired particle type as a bullet by default
	fired_particle_type = "bullet"

	new pos, bool:particle_is_fired = xs_vec_equal(fired_particle_start_origin, start)

	if (particle_is_fired) {
		// trace start origin is equal to the fired particle start origin

		// the particle is fired

		if (SHOTGUNS_BIT_SUM & (1<<weapon_id)) {
			// the current weapon is a shotgun

			if (gametime != g_last_shotgun_shot_gametime[id]) {
				// the current game time isn't equal to the last shotgun shot game time

				// this is a new shotgun shot

				// set last shotgun shot game time to the current game time
				g_last_shotgun_shot_gametime[id] = gametime

				// this is a fisrt particle of the shotgun shot

				// set number of shotgun shot particle to 1
				g_shotgun_shot_particle_number[id] = 1
			}
			else {
				// the current game time is equal to the last shotgun shot game time

				// this is a next particle from the same shotgun shot

				// increase number of shotgun shot particle + 1
				g_shotgun_shot_particle_number[id]++
			}

			// the fired particle type is a shotgun shot particle
			fired_particle_type = "shotgun shot particle"

			// shotgun shot particle #N is fired
			pos += formatex(info[pos], sizeof info - pos - 1, "[%6.6f] %s #%d is fired", gametime, fired_particle_type, g_shotgun_shot_particle_number[id])
		}
		else {
			// the current weapon isn't a shotgun

			// bullet is fired
			pos += formatex(info[pos], sizeof info - pos - 1, "[%6.6f] %s is fired", gametime, fired_particle_type)
		}
	}

	// get an index of the entity that is hit by the fired particle
	new ent = get_tr2(ptr, TR_pHit)

	if (is_user_alive(ent)) {
		// the hit object of the fired particle is an alive player

		// the fired particle is hit a player with the client index #N

		if (particle_is_fired)
			pos += formatex(info[pos], sizeof info - pos - 1, ", the %s is hit a player #%d", fired_particle_type, ent)
		else
			pos += formatex(info[pos], sizeof info - pos - 1, "[%6.6f] the %s is hit a player #%d", gametime, fired_particle_type, ent)
	}
	else {
		// the hit object of the fired particle isn't an alive player

		// the fired particle is collide with the world/brush obstacle

		if (particle_is_fired)
			pos += formatex(info[pos], sizeof info - pos - 1, ", %s is collide with an obstacle", fired_particle_type)
		else
			pos += formatex(info[pos], sizeof info - pos - 1, "[%6.6f] %s is collide with an obstacle", gametime, fired_particle_type)
	}

	// get the end position of the current trace
	get_tr2(ptr, TR_vecEndPos, vector)

	//new Float:vector2[3]
	// get the actual vector of the current trace
	//xs_vec_sub(vector, start, vector2)

	// get the current actual vector of the fired particle
	xs_vec_sub(vector, fired_particle_start_origin, vector)

	// add that vector value to the gathered information
	formatex(info[pos], sizeof info - pos - 1, ", the current vector is {%.1f, %.1f, %.1f}", vector[0], vector[1], vector[2])

	// output all gathered information of the fired particle to the server console
	server_print("%s", info)

	return FMRES_IGNORED
}
